from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_cors import CORS
import logging
from app.config.config import config, current_env
import os

db = SQLAlchemy()

def create_app():
    """Create and configure the Flask application"""
    app = Flask(__name__)
    
    # 加载配置
    app.config.from_object(config[current_env])
    
    # 初始化扩展
    CORS(app)
    db.init_app(app)
    
    # 注册蓝图
    from app.api.meta_data import bp as meta_bp
    from app.api.data_resource import bp as resource_bp
    from app.api.data_model import bp as data_model_bp
    from app.api.data_interface import bp as data_interface_bp
    from app.api.data_metric import bp as data_metric_bp
    from app.api.production_line import bp as production_line_bp
    from app.api.graph import bp as graph_bp
    from app.api.system import bp as system_bp
    
    app.register_blueprint(meta_bp, url_prefix='/api/meta')
    app.register_blueprint(resource_bp, url_prefix='/api/resource')
    app.register_blueprint(data_model_bp, url_prefix='/api/model')
    app.register_blueprint(data_interface_bp, url_prefix='/api/interface')
    app.register_blueprint(data_metric_bp, url_prefix='/api/metric')
    app.register_blueprint(production_line_bp, url_prefix='/api/pipeline')
    app.register_blueprint(graph_bp, url_prefix='/api/graph')
    app.register_blueprint(system_bp, url_prefix='/api/system')
    
    # Configure logging
    configure_logging(app)
    
    # 输出启动信息
    app.logger.info(f"Starting server in {current_env} mode on port {app.config['PORT']}")
    
    return app

def configure_logging(app):
    """Configure logging for the application"""
    env = os.environ.get('FLASK_ENV', 'development')
    log_file = f'flask_{env}.log'
    
    logger = logging.getLogger("app")
    handler = logging.FileHandler(log_file, encoding='UTF-8')
    handler.setLevel(logging.INFO)
    logging_format = logging.Formatter(
        '%(asctime)s - %(levelname)s - %(filename)s - %(funcName)s - %(lineno)s - %(message)s')
    handler.setFormatter(logging_format)
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    console.setFormatter(logging_format)
    logger.addHandler(handler)
    logger.addHandler(console)
    
    return logger
