"""
元数据处理模块
提供元数据管理、查询、图谱分析和非结构化数据处理的核心功能
"""

import time
import logging
from app.services.neo4j_driver import neo4j_driver
import ast
import re
import pandas as pd
from minio import S3Error
from py2neo import Relationship
import json
import io
import random
import string
import numpy as np
from openai import OpenAI
from flask import current_app
from app.core.llm.llm_service import llm_client as llm_call  # 导入core/llm模块的函数

logger = logging.getLogger("app")

def get_formatted_time():
    """获取格式化的当前时间"""
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())

def translate_and_parse(content):
    """
    翻译内容并返回结果
    
    Args:
        content: 需要翻译的内容
        
    Returns:
        list: 包含翻译结果的列表
    """
    # 调用LLM服务进行翻译
    translated_text = llm_call(content)
    
    # 如果翻译失败，返回原文
    if translated_text is None:
        return [content]
    
    # 确保返回格式为列表
    return [translated_text]

# 为保持原有功能，保留旧的llm_client函数
def llm_client(content):
    """调用LLM服务进行中英文翻译，返回结果"""
    # 直接调用基础服务层的翻译函数
    return llm_call(content)

def infer_column_type(df):
    try:
        # 列名
        res = df.columns.to_list()
        columns = ','.join(res)
        
        # 使用配置中的LLM参数
        api_k = current_app.config.get('LLM_API_KEY')
        base_u = current_app.config.get('LLM_BASE_URL')
        model = current_app.config.get('LLM_MODEL_NAME')
        
        client = OpenAI(api_key=api_k, base_url=base_u)
        response = client.chat.completions.create(
            model=model,
            messages=[
                {"role": "system", "content": "你是一个PostgreSQL数据库专家，精通PostgreSQL所有数据类型和最佳实践"},
                {"role": "user",
                 "content": "请根据以下数据表内容：" + str(df.head(n=6)) + "其列名为" + columns +
                            "，帮我判断每个列最合适的PostgreSQL数据类型。请注意以下要求：" +
                            "1. 对于文本数据，使用varchar并给出合适长度，如varchar(50)、varchar(255)等" +
                            "2. 对于整数，根据数值范围选择smallint、integer或bigint" +
                            "3. 对于小数，如果是金额相关字段使用numeric(15,2)，其他小数使用numeric(18,6)" +
                            "4. 对于日期时间，根据实际情况选择date、time或timestamp" +
                            "5. 对于布尔值，使用boolean类型" +
                            "6. 如果是JSON数据，使用jsonb类型" +
                            "请以列表格式返回，列表中的元素顺序要与输入的列名顺序一致，如：" +
                            "['varchar(255)', 'integer', 'numeric(15,2)', 'timestamp']" +
                            "只返回列表，不要有任何其他说明文字"}
            ],
            max_tokens=1024,
            temperature=0.1,
            stream=False
        )
        result = response.choices[0].message.content
        res = result.strip('`').strip('python').strip('`').strip()
        
        # 使用 ast.literal_eval 函数将字符串转换为列表
        result_list = ast.literal_eval(res)
        return result_list
    except Exception as e:
        logger.error(f"列类型推断失败: {str(e)}")
        # 返回一个空列表或默认类型列表，保持返回类型一致
        return ['varchar(255)'] * len(df.columns) if not df.empty else []

def meta_list(page, page_size, search="", en_name_filter=None, 
             name_filter=None, category_filter=None, time_filter=None, tag_filter=None):
    """
    获取元数据列表
    
    Args:
        page: 当前页码
        page_size: 每页数量
        search: 搜索关键词
        en_name_filter: 英文名称过滤
        name_filter: 名称过滤
        category_filter: 分类过滤
        time_filter: 时间过滤
        tag_filter: 标签过滤
    
    Returns:
        tuple: (result_list, total_count)
    """
    try:
        with neo4j_driver.get_session() as session:
            # 构建查询条件
            match_clause = "MATCH (n:Metadata)"
            where_conditions = []
            
            if search:
                where_conditions.append(f"n.name CONTAINS '{search}'")
            
            if en_name_filter:
                where_conditions.append(f"n.en_name CONTAINS '{en_name_filter}'")
                
            if name_filter:
                where_conditions.append(f"n.name CONTAINS '{name_filter}'")
                
            if category_filter:
                where_conditions.append(f"n.category = '{category_filter}'")
                
            if time_filter:
                where_conditions.append(f"n.createTime CONTAINS '{time_filter}'")
                
            if tag_filter:
                where_conditions.append(f"n.tag = '{tag_filter}'")
            
            where_clause = " WHERE " + " AND ".join(where_conditions) if where_conditions else ""
            
            # 计算总数
            count_cypher = f"{match_clause}{where_clause} RETURN count(n) as count"
            count_result = session.run(count_cypher)
            total_count = count_result.single()["count"]
            
            # 分页查询
            skip = (page - 1) * page_size
            cypher = f"""
            {match_clause}{where_clause}
            RETURN n
            ORDER BY n.name
            SKIP {skip} LIMIT {page_size}
            """
            result = session.run(cypher)
            
            # 格式化结果
            result_list = []
            for record in result:
                node = dict(record["n"])
                node["id"] = record["n"].id
                result_list.append(node)
                
            return result_list, total_count
    except Exception as e:
        logger.error(f"获取元数据列表失败: {str(e)}")
        raise

def handle_id_unstructured(node_id):
    """处理非结构化数据节点"""
    try:
        with neo4j_driver.get_session() as session:
            query = "MATCH (n) WHERE id(n) = $node_id RETURN n"
            result = session.run(query, node_id=int(node_id))
            node = result.single()
            if node:
                return dict(node["n"])
            else:
                return None
    except Exception as e:
        logger.error(f"处理非结构化数据节点失败: {str(e)}")
        raise

def get_file_content(minio_client, bucket_name, object_name):
    """从MinIO获取文件内容"""
    try:
        # 获取对象
        response = minio_client.get_object(bucket_name, object_name)
        
        # 读取内容
        file_content = response.read().decode('utf-8')
        return file_content
    except S3Error as e:
        logger.error(f"MinIO访问失败: {str(e)}")
        raise
    finally:
        response.close()
        response.release_conn()

def parse_text(text):
    """解析文本内容，提取关键信息"""
    # 提取作者信息
    author_match = re.search(r'作者[：:]\s*(.+?)[\n\r]', text)
    author = author_match.group(1) if author_match else ""
    
    # 提取关键词
    keyword_match = re.search(r'关键词[：:]\s*(.+?)[\n\r]', text)
    keywords = keyword_match.group(1) if keyword_match else ""
    
    return {
        "author": author.strip(),
        "keywords": keywords.strip()
    }

def parse_keyword(content):
    """解析关键词"""
    if "，" in content:
        return content.split("，")
    elif "," in content:
        return content.split(",")
    elif "、" in content:
        return content.split("、")
    else:
        return [content]

def text_resource_solve(receiver, name, keyword):
    """处理文本资源解析"""
    try:
        # 构建提示词 - 使用简短明确的指令
        prompt = f"{name}"
        
        # 调用LLM获取英文翻译
        english_name = llm_client(prompt)
        
        # 提取关键词
        keywords = parse_keyword(keyword)
        
        # 为每个关键词获取英文翻译
        keywords_en = []
        for kw in keywords:
            # 直接使用关键词作为翻译输入
            kw_en = llm_client(kw)
            keywords_en.append(kw_en)
            
        # 构建返回数据
        return {
            "name": name,
            "en_name": english_name,
            "keywords": keywords,
            "keywords_en": keywords_en
        }
    except Exception as e:
        logger.error(f"文本资源处理失败: {str(e)}")
        raise

def meta_kinship_graph(node_id):
    """
    获取元数据亲缘关系图谱
    
    Args:
        node_id: 元数据节点ID
    
    Returns:
        dict: 图谱数据
    """
    try:
        with neo4j_driver.get_session() as session:
            # 获取节点和直接关系
            cypher = """
            MATCH (n)-[r]-(m)
            WHERE id(n) = $node_id
            RETURN n, r, m
            """
            result = session.run(cypher, node_id=int(node_id))
            
            # 格式化结果为图谱数据
            nodes = {}
            relationships = []
            
            for record in result:
                # 处理源节点
                source_node = dict(record["n"])
                source_node["id"] = record["n"].id
                nodes[source_node["id"]] = source_node
                
                # 处理目标节点
                target_node = dict(record["m"])
                target_node["id"] = record["m"].id
                nodes[target_node["id"]] = target_node
                
                # 处理关系
                rel = record["r"]
                relationship = {
                    "id": rel.id,
                    "source": record["n"].id,
                    "target": record["m"].id,
                    "type": rel.type
                }
                relationships.append(relationship)
            
            # 转换为列表
            nodes_list = list(nodes.values())
            
            return {
                "nodes": nodes_list,
                "relationships": relationships
            }
    except Exception as e:
        logger.error(f"获取元数据亲缘关系图谱失败: {str(e)}")
        raise

def meta_impact_graph(node_id):
    """
    获取元数据影响关系图谱
    
    Args:
        node_id: 元数据节点ID
    
    Returns:
        dict: 图谱数据
    """
    try:
        with neo4j_driver.get_session() as session:
            # 获取所有可达节点和关系
            cypher = """
            MATCH path = (n)-[*1..3]-(m)
            WHERE id(n) = $node_id
            RETURN path
            """
            result = session.run(cypher, node_id=int(node_id))
            
            # 格式化结果
            nodes = {}
            relationships = set()
            
            for record in result:
                path = record["path"]
                
                # 处理路径中的所有节点
                for node in path.nodes:
                    node_dict = dict(node)
                    node_dict["id"] = node.id
                    nodes[node.id] = node_dict
                
                # 处理路径中的所有关系
                for rel in path.relationships:
                    relationship = (rel.id, rel.start_node.id, rel.end_node.id, rel.type)
                    relationships.add(relationship)
            
            # 转换为列表
            nodes_list = list(nodes.values())
            relationships_list = [
                {
                    "id": rel[0],
                    "source": rel[1],
                    "target": rel[2],
                    "type": rel[3]
                }
                for rel in relationships
            ]
            
            return {
                "nodes": nodes_list,
                "relationships": relationships_list
            }
    except Exception as e:
        logger.error(f"获取元数据影响关系图谱失败: {str(e)}")
        raise

def parse_entity_relation(text):
    """从文本中解析实体关系"""
    try:
        # 构建提示词
        prompt = f"""
        请从以下文本中提取实体及其关系，以JSON格式返回，格式为:
        [
          {{"entity1": "实体1", "relation": "关系", "entity2": "实体2"}}
        ]
        
        文本内容:
        {text}
        """
        
        # 调用LLM获取关系提取结果
        result = llm_client(prompt)
        
        # 解析JSON结果
        try:
            relations = json.loads(result)
            return relations
        except json.JSONDecodeError:
            logger.error(f"关系提取结果JSON解析失败: {result}")
            return []
            
    except Exception as e:
        logger.error(f"实体关系提取失败: {str(e)}")
        return []
        
def handle_txt_graph(node_id, entity, entity_en):
    """处理文本图谱创建"""
    try:
        # 创建实体节点
        with neo4j_driver.get_session() as session:
            # 查找源节点
            query = "MATCH (n) WHERE id(n) = $node_id RETURN n"
            result = session.run(query, node_id=int(node_id))
            source_node = result.single()["n"]
            
            if not source_node:
                return False
                
            # 创建实体节点
            cypher = """
            MERGE (e:Entity {name: $name, en_name: $en_name})
            ON CREATE SET e.createTime = $create_time
            RETURN e
            """
            
            create_time = get_formatted_time()
            result = session.run(
                cypher,
                name=entity,
                en_name=entity_en,
                create_time=create_time
            )
            
            entity_node = result.single()["e"]
            
            # 创建关系
            if source_node and entity_node:
                # 检查关系是否已存在
                rel_check = """
                MATCH (s)-[r:CONTAINS]->(e) 
                WHERE id(s) = $source_id AND id(e) = $entity_id
                RETURN r
                """
                
                rel_result = session.run(
                    rel_check,
                    source_id=source_node.id,
                    entity_id=entity_node.id
                )
                
                # 如果关系不存在，则创建
                if not rel_result.single():
                    rel_create = """
                    MATCH (s), (e) 
                    WHERE id(s) = $source_id AND id(e) = $entity_id
                    CREATE (s)-[r:CONTAINS]->(e)
                    RETURN r
                    """
                    
                    session.run(
                        rel_create,
                        source_id=source_node.id,
                        entity_id=entity_node.id
                    )
                    
            return True
    except Exception as e:
        logger.error(f"文本图谱处理失败: {str(e)}")
        return False

def solve_unstructured_data(node_id, minio_client, prefix):
    """处理非结构化数据并提取实体关系"""
    try:
        # 获取节点数据
        node_data = handle_id_unstructured(node_id)
        if not node_data:
            logger.error(f"节点不存在: {node_id}")
            return False
            
        # 获取对象路径
        object_name = node_data.get('url')
        if not object_name:
            logger.error(f"文档路径不存在: {node_id}")
            return False
            
        # 获取文件内容
        file_content = get_file_content(minio_client, bucket_name=node_data.get('bucket_name', 'dataops'), object_name=object_name)
        
        # 解析文本内容中的实体关系
        relations = parse_entity_relation(file_content[:5000])  # 只处理前5000字符，避免过大内容
        
        # 如果成功提取了关系
        if relations:
            # 更新节点信息
            with neo4j_driver.get_session() as session:
                update_cypher = """
                MATCH (n) WHERE id(n) = $node_id
                SET n.processed = true, n.processTime = $process_time
                RETURN n
                """
                
                process_time = get_formatted_time()
                session.run(
                    update_cypher,
                    node_id=int(node_id),
                    process_time=process_time
                )
                
                # 为每个提取的关系创建实体和关系
                for relation in relations:
                    entity1 = relation.get("entity1", "")
                    relation_type = relation.get("relation", "")
                    entity2 = relation.get("entity2", "")
                    
                    if entity1 and entity2 and relation_type:
                        # 翻译实体名称为英文 - 使用简短直接的输入
                        entity1_en = llm_client(entity1)
                        entity2_en = llm_client(entity2)
                        
                        # 创建第一个实体
                        entity1_cypher = """
                        MERGE (e:Entity {name: $name})
                        ON CREATE SET e.en_name = $en_name, e.createTime = $create_time
                        RETURN e
                        """
                        
                        entity1_result = session.run(
                            entity1_cypher,
                            name=entity1,
                            en_name=entity1_en,
                            create_time=process_time
                        )
                        entity1_node = entity1_result.single()["e"]
                        
                        # 创建第二个实体
                        entity2_cypher = """
                        MERGE (e:Entity {name: $name})
                        ON CREATE SET e.en_name = $en_name, e.createTime = $create_time
                        RETURN e
                        """
                        
                        entity2_result = session.run(
                            entity2_cypher,
                            name=entity2,
                            en_name=entity2_en,
                            create_time=process_time
                        )
                        entity2_node = entity2_result.single()["e"]
                        
                        # 创建它们之间的关系
                        rel_cypher = """
                        MATCH (e1:Entity), (e2:Entity)
                        WHERE id(e1) = $entity1_id AND id(e2) = $entity2_id
                        MERGE (e1)-[r:`{relation_type}`]->(e2)
                        RETURN r
                        """.replace("{relation_type}", relation_type)
                        
                        session.run(
                            rel_cypher,
                            entity1_id=entity1_node.id,
                            entity2_id=entity2_node.id
                        )
                        
                        # 创建源节点与实体的关系
                        source_rel1_cypher = """
                        MATCH (s), (e:Entity)
                        WHERE id(s) = $source_id AND id(e) = $entity_id
                        MERGE (s)-[r:CONTAINS]->(e)
                        RETURN r
                        """
                        
                        session.run(
                            source_rel1_cypher,
                            source_id=int(node_id),
                            entity_id=entity1_node.id
                        )
                        
                        source_rel2_cypher = """
                        MATCH (s), (e:Entity)
                        WHERE id(s) = $source_id AND id(e) = $entity_id
                        MERGE (s)-[r:CONTAINS]->(e)
                        RETURN r
                        """
                        
                        session.run(
                            source_rel2_cypher,
                            source_id=int(node_id),
                            entity_id=entity2_node.id
                        )
            
            return True
        else:
            logger.warning(f"未能从文本中提取到实体关系: {node_id}")
            return False
            
    except Exception as e:
        logger.error(f"处理非结构化数据失败: {str(e)}")
        return False 